;; Evaluate Python code with result output -*- lexical-binding: t -*-

(require 'json)
(require 'python)

;; From https://bugs.python.org/issue39820#msg387189
;;
;; How can we hide __PYTHON_EL_eval from stack traces?  This is a
;; problem in `python-shell-send-*' as well, see
;; https://debbugs.gnu.org/cgi/bugreport.cgi?bug=32042
(defvar python-shell-eval-setup-code
  "\
def __PYTHON_EL_eval(source, filename):
    print()
    import ast
    g, p = globals(), ast.parse(source, filename)
    expr, res = None, None
    if p.body and isinstance(p.body[-1], ast.Expr):
        expr = p.body.pop()
    code = compile(p, filename, 'exec')
    exec(code, g, g)
    if expr:
        code = compile(ast.Expression(expr.value), filename, 'eval')
        res = eval(code, g, g)
    return res"
  "Code used to evaluate statements in inferior Python processes.")

;;;###autoload
(defun python-shell-eval-string (string &optional file-name add-history process)
  "Send STRING to inferior Python PROCESS."
  (interactive
   (list (read-string "Python statement: ") nil nil nil))
  (unless process (setq process (python-shell-get-process-or-error t)))
  (unless file-name (setq file-name "<string>"))
  (with-current-buffer (process-buffer process)
    (compilation-forget-errors)
    (when add-history
      (comint-add-to-input-history
       (if (stringp add-history) add-history string))))
  (comint-send-string process
                      (format "exec(%s);__PYTHON_EL_eval(%s, %s)\n"
                              (json-encode python-shell-eval-setup-code)
                              (json-encode string)
                              (json-encode file-name))))

;;;###autoload
(defun python-shell-eval-region (start end &optional add-history)
  "Send the region delimited by START and END to inferior Python process."
  (interactive
   (list (region-beginning) (region-end) (not current-prefix-arg)))
  (python-shell-eval-string (python-shell-buffer-substring start end)
                            (buffer-file-name)
                            (buffer-substring-no-properties start end)))

;;;###autoload
(defun python-shell-eval-buffer (&optional send-main)
  "Send buffer to inferior Python process."
  (interactive "P")
  (let ((string (python-shell-buffer-substring (point-min) (point-max) (not send-main))))
    (python-shell-eval-string string (buffer-file-name))))

(provide 'python-shell-eval)
;;; python-shell-eval.el ends here
